//***************************************************************************
//
//   abstract class calc (from MDS v.5.2)
//   base method of calculation with class model
//   (c) 2002-2004 Dr.Bashnyakov A. 
//   (bash@unicyb.kiev.ua)
//
//***************************************************************************
//                                                                         
//   This program is free software; you can redistribute it and/or modify  
//   it under the terms of the GNU General Public License as published by  
//   the Free Software Foundation; either version 2 of the License, or     
//   (at your option) any later version.                                   
//                                                                         
//***************************************************************************


#ifndef _calc_h_
#define _calc_h_


#include "model.h"
#include "region.h"


// base class of calculation methods
template <class T = MDS_DEFAULT_TYPE, class EXCEPTION = MDS_DEFAULT_EXCEPTION>
class calc  
{
public:
        calc()
        {
                dummyRes.density = 0;
        }

// general calculation function: must be overloaded in new calculation class
        virtual std::valarray<T> Method(const std::valarray<model<T,EXCEPTION> *>& pModels, 
                const cauchy_region<T,EXCEPTION>& region, result_manager<T,T,EXCEPTION>& resCalc, bool isEnlargedPhase) = 0;

// general calculation interface
        std::valarray<T> Calculate(const std::valarray<model<T,EXCEPTION> *>& pModels, 
                const cauchy_region<T,EXCEPTION>& region, result_manager<T,T,EXCEPTION>& resCalc, bool isEnlargedPhase = false) 
        {
                return Method(pModels, region, resCalc, isEnlargedPhase); 
        }

// alt calculation interface: without result_manager         
        std::valarray<T> Calculate(const std::valarray<model<T,EXCEPTION> *>& pModels, 
                const cauchy_region<T,EXCEPTION>& region, bool isEnlargedPhase = false) 
        {
                return Method(pModels, region, dummyRes, isEnlargedPhase);
        }

// alt calculation interface: only one model         
        std::valarray<T> Calculate(model<T,EXCEPTION>* pModel, 
                const cauchy_region<T,EXCEPTION>& region, result_manager<T,T,EXCEPTION>& resCalc, bool isEnlargedPhase = false) 
        {
                return Method(std::valarray<model<T,EXCEPTION> *>(pModel, 1), region, resCalc, isEnlargedPhase);
        }

        std::valarray<T> Calculate(model<T,EXCEPTION>& OneModel, 
                const cauchy_region<T,EXCEPTION>& region, result_manager<T,T,EXCEPTION>& resCalc, bool isEnlargedPhase = false) 
        {
                return Method(std::valarray<model<T,EXCEPTION> *>(&OneModel, 1), region, resCalc, isEnlargedPhase);
        }

        std::valarray<T> Calculate( model<T,EXCEPTION>* pModel, const cauchy_region<T,EXCEPTION>& region, bool isEnlargedPhase = false) 
        {
                return Method( std::valarray<model<T,EXCEPTION> *>(pModel, 1), region, dummyRes, isEnlargedPhase );
        }

        std::valarray<T> Calculate(model<T,EXCEPTION>& OneModel, 
                const cauchy_region<T,EXCEPTION>& region, bool isEnlargedPhase = false) 
        {
                return Method(std::valarray<model<T,EXCEPTION> *>(&OneModel, 1), region, dummyRes, isEnlargedPhase);
        }

protected:
// recomended service: returning results
        std::valarray<T> CalcReturnValue(const T& res_t,
                const std::valarray<T>& res_x, bool isEnlargedRes)
        {
                if (isEnlargedRes)
                {
                        size_t cnt, dim = res_x.size();

                        std::valarray<T> v(dim + 1);

                        for (cnt = 0; cnt < dim; cnt++) v[cnt + 1] = res_x[cnt];

                        v[0] = res_t;
                        
                        return v;
                }

                return res_x;
        }


private:
        result_manager<T, T, EXCEPTION> dummyRes;  // dummy container
};


#endif  
