//***************************************************************************
//
//   class dynamo (from MDS v.5.2)
//   modeling dynamic of algebraic system(s) y=f(t)
//   (c) 2002-2004 Dr.Bashnyakov A. 
//   (bash@unicyb.kiev.ua)
//
//***************************************************************************
//                                                                         
//   This program is free software; you can redistribute it and/or modify  
//   it under the terms of the GNU General Public License as published by  
//   the Free Software Foundation; either version 2 of the License, or     
//   (at your option) any later version.                                   
//                                                                         
//***************************************************************************


#ifndef _dynamo_h_
#define _dynamo_h_


#include "calc.h"


// base class of modeling methods
template <class T = MDS_DEFAULT_TYPE, class EXCEPTION = MDS_DEFAULT_EXCEPTION>
class dynamo : public calc<T, EXCEPTION> 
{
public:
        explicit dynamo(const T& userStep) 
                : step(userStep)
        {}

// general modeling function
        virtual std::valarray<T> Method(const std::valarray<model<T,EXCEPTION> *>& pModels, 
                const cauchy_region<T,EXCEPTION>& region, result_manager<T,T,EXCEPTION>& resCalc, bool isEnlargedPhase) 
        {
                T modelTime, modelStep = step, tStart = region.GetStartTime(), tFinal = region.GetFinalTime();

                if (!region.isFuncFinalCondition())
                        if( (tStart > tFinal && tStart < tStart + modelStep) || 
                                (tStart < tFinal && tStart > tStart + modelStep) ) 
                                        modelStep = -modelStep;

                size_t dim = model<T,EXCEPTION>::GetDimension(pModels);

                std::valarray<T> x(dim), y(dim);

                modelTime = tStart;

                model<T, EXCEPTION>::DoSystem(pModels, x, modelTime);

                y = x;

                while (region.isRegion(modelTime, x) >= 0)
                {  
                        resCalc.Add(modelTime, x, false);

                        modelTime += modelStep;

                        y = x;

                        model<T, EXCEPTION>::DoSystem(pModels, x, modelTime);
                }

                return CalcReturnValue(modelTime - modelStep, y, isEnlargedPhase);
        }


// parameters
        T step;  // modeling step
};


#endif  
