//***************************************************************************
//
//   class intergation (from MDS v.5.2)
//   integration of differential equation system(s) by Euler method
//   (c) 2002-2004 Dr.Bashnyakov A. 
//   (bash@unicyb.kiev.ua)
//
//***************************************************************************
//                                                                         
//   This program is free software; you can redistribute it and/or modify  
//   it under the terms of the GNU General Public License as published by  
//   the Free Software Foundation; either version 2 of the License, or     
//   (at your option) any later version.                                   
//                                                                         
//***************************************************************************


#ifndef _integr_h_
#define _integr_h_


#include "calc.h"


// base class of integration methods - Euler method
template <class T = MDS_DEFAULT_TYPE, class EXCEPTION = MDS_DEFAULT_EXCEPTION>
class integration : public calc<T, EXCEPTION> 
{
public:
        explicit integration(const T& userStep) 
                : step(userStep)
        {}

// general integration function: must be overloaded in new integration class
        virtual std::valarray<T> Method( const std::valarray<model<T,EXCEPTION> *>& pModels, 
                const cauchy_region<T,EXCEPTION>& region, result_manager<T,T,EXCEPTION>& resCalc, bool isEnlargedPhase) 
        {
                T modelTime, modelStep = step, tStart = region.GetStartTime(), tFinal = region.GetFinalTime();

                if (!region.isFuncFinalCondition())
                        if ( (tStart > tFinal && tStart < tStart + modelStep) || 
                                (tStart < tFinal && tStart > tStart + modelStep) ) 
                                        modelStep = -modelStep;

                size_t dim = model<T,EXCEPTION>::GetDimension(pModels);

                std::valarray<T> x(dim), z(dim), y(dim);

                region.GetCauchyCondition(modelTime, x);

                y = x;

                MDS_ASSERT(dim == region.GetCauchyConditionSize(), EXCEPTION,
                        "[integration] incorrect Cauchy condition or quantity of models in Calculation(..)");
                
                while (region.isRegion(modelTime, x) >= 0)
                {  
                        resCalc.Add(modelTime, x, false);

                        model<T, EXCEPTION>::DoSystem(pModels, z, modelTime, x);

                        y = x;

                        x += modelStep * z;

                        modelTime += modelStep;
                }

                return CalcReturnValue(modelTime - modelStep, y, isEnlargedPhase);
        }


// parameters
        T step;  // integration step

};


#endif 
