//***************************************************************************
//
//   MDS v.5.2
//   MDS is Modeling of Dynamical System
//   includes:
//     1. abstract class model (equations of model)
//     2. class cauchy_region (encapsulating Cauchy condition for integration)
//     3. abstract class calc (base class for calculation)
//     4. class dynamo (for modeling dynamic of algebraic model)
//     5. class discrete (for difference scheme)
//     6. class integration (for differential equation)
//     7. class integration_rk45 (for differential equation)
//     8. class result_manager (collecting and saving calculation results)
//     9. class mds_exception, mds_exception_msg (control of operations)
//
//   (c) 2002-2004 Dr.Bashnyakov A. (bash@unicyb.kiev.ua)
//   (c) 2003-2004 Polishchuk A.(zagamuf@ua.fm), class integration_rk45
//
//   ------------------------------------------------------------------------
//                              Compilation
//   I.UNIX
//   g++ main.cpp -otestmds
//   (use D_DEBUG option for debugging)
//   for gnu c++ v.2.96 you MUST include also gnu_std_valarray.h in mds.h
//
//   II.Windows
//   Use Microsoft VisualC++ and open main.dsw
//   (use Debug configuration for debugging)
//                     -or-
//   Use Borland C++Builder and open Project.bpr
//   (use Full Debug configuration for debugging)
//
//***************************************************************************
//                                                                         
//   This program is free software; you can redistribute it and/or modify  
//   it under the terms of the GNU General Public License as published by  
//   the Free Software Foundation; either version 2 of the License, or     
//   (at your option) any later version.                                   
//                                                                         
//***************************************************************************

// General include for using mds
// Of course we can include each of those files manually                                                                        


#ifndef _mds_h_
#define _mds_h_


#include "mds_error.h"
#include "mds_assert.h"


typedef double MDS_DEFAULT_TYPE;  // base type for calcualation: we can change it
typedef mds_exception_message MDS_DEFAULT_EXCEPTION; // base type for exception: we can change it


#include "model.h"
#include "region.h"
#include "calc.h"
#include "dynamo.h"
#include "discrete.h"
#include "integr.h"
#include "rk45.h"
#include "result.h"

// patch for gnu c++ 2.96:
//#include "gnu_std_valarray.h"


typedef model<MDS_DEFAULT_TYPE, MDS_DEFAULT_EXCEPTION> Model;
typedef cauchy_region<MDS_DEFAULT_TYPE, MDS_DEFAULT_EXCEPTION> CauchyRegion;
typedef dynamo<MDS_DEFAULT_TYPE, MDS_DEFAULT_EXCEPTION> Dynamo;
typedef discrete<MDS_DEFAULT_TYPE, MDS_DEFAULT_EXCEPTION> Discrete;
typedef integration<MDS_DEFAULT_TYPE, MDS_DEFAULT_EXCEPTION> Integration;
typedef integration_rk45<MDS_DEFAULT_TYPE, MDS_DEFAULT_EXCEPTION> IntegrationRK45;
typedef result_manager<MDS_DEFAULT_TYPE, MDS_DEFAULT_TYPE, MDS_DEFAULT_EXCEPTION> ResultManager;


#endif 
