//***************************************************************************
//
//   class intergation (from MDS v.5.2)
//   integration of differential equation system(s) by Runge-Kutta 4(5) method
//   (c) 2003, 2004 Polishchuk A.
//   (zagamuf@ua.fm) 
//
//***************************************************************************
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//***************************************************************************


#ifndef _rk45_h_
#define _rk45_h_


#include "calc.h"


template <class T = MDS_DEFAULT_TYPE, class EXCEPTION = MDS_DEFAULT_EXCEPTION>
class integration_rk45 : public calc<T, EXCEPTION>
{
public:
        explicit integration_rk45(const T& userStep)
                : step(userStep)
        {}

        virtual std::valarray<T> Method(const std::valarray<model<T,EXCEPTION> *>& pModels,
                const cauchy_region<T,EXCEPTION>& pRegion, result_manager<T,T,EXCEPTION>& pResultManager, bool isEnlargedPhase)
        {
                T modelTime, modelStep = step, tStart = pRegion.GetStartTime(), tFinal = pRegion.GetFinalTime();

                if (!pRegion.isFuncFinalCondition())
                        if ((tStart > tFinal && tStart < tStart+modelStep) ||
                                (tStart < tFinal && tStart > tStart+modelStep))
                                        modelStep = -modelStep;

                size_t dim = model<T,EXCEPTION>::GetDimension(pModels);

                std::valarray<T> x(dim), y(dim), k0(dim), k1(dim), k2(dim), k3(dim);

                pRegion.GetCauchyCondition(modelTime, x);
                
                y = x;

                MDS_ASSERT(dim == pRegion.GetCauchyConditionSize(), EXCEPTION,
                        "[integration_rk45] incorrect Cauchy condition or quantity of models in Calculation(..)");

                while (pRegion.isRegion(modelTime, x) >= 0)
                {
                        pResultManager.Add(modelTime, x, false);

                        model<T,EXCEPTION>::DoSystem(pModels, k0, modelTime, x);

                        k0 *= modelStep;

                        model<T,EXCEPTION>::DoSystem(pModels, k1, modelTime + modelStep / T(2), x + k0 / T(2));

                        k1 *= modelStep;

                        model<T,EXCEPTION>::DoSystem(pModels, k2, modelTime + modelStep / T(2), x + k1 / T(2));

                        k2 *= modelStep;

                        model<T,EXCEPTION>::DoSystem(pModels, k3, modelTime + modelStep, x + k2);

                        k3 *= modelStep;

                        y = x;

                        x += (k0 + T(2) * k1 + T(2) * k2 + k3) / T(6);

                        modelTime += modelStep;
                }

                return CalcReturnValue(modelTime - modelStep, y, isEnlargedPhase);
        }

        T step;
};


#endif  
