//***************************************************************************
//
//                   Example of modeling with MDS (v.5.2)
//
//   MDS is Modeling of Dynamical System
//   includes:
//     1. abstract class model (equations of model)
//     2. class cauchy_region (encapsulating Cauchy condition for integration)
//     3. abstract class calc (base class for calculation)
//     4. class dynamo (for modeling dynamic of algebraic model)
//     5. class discrete (for difference scheme)
//     6. class integration (for differential equation)
//     7. class integration_rk45 (for differential equation)
//     8. class result_manager (collecting and saving calculation results)
//     9. class mds_exception, mds_exception_message (control of operations)
//
//   It is good idea to use modules not included in MDS:
//   GetData (reading ini-files), class ProgLog (logging process), 
//   class matrix (very powerful) etc.
//
//
//   ------------------------------------------------------------------------
//                              Compilation
//   I.UNIX
//   g++ main.cpp -otestmds
//   (use -D_DEBUG option for debugging)
//   for gnu c++ v.2.96 you MUST include also gnu_std_valarray.h in mds.h
//
//   II.Windows
//   Use Microsoft VisualC++ and open main.dsw
//   (use Debug configuration for debugging)
//                     -or-
//   Use Borland C++Builder and open Project.bpr
//   (use Full Debug configuration for debugging)
//
//***************************************************************************


//********** 1-ST STEP: including necessary h-files for using MDS
// WARNING: for gnu c++ v.2.96 you MUST include also gnu_std_valarray.h in mds.h
#include "../mds/mds.h"

// additional includes:
#include <math.h>
#include <valarray> 
using namespace std;


//********** 2-ST STEP: model with 2 diff. equations:
// x1'(t)=f1(t,x)=cos(t + 1) 
// x2'(t)=f2(t,x)=x1 + t

class SimpleModel : public Model
{
public:

SimpleModel() : Model(2)  // e.g. dimension is 2: f1, f2    
{ /* do additional initialization */ }

// we MUST overload function of right part of system
// func has dimension 2 (according model dimension) and takes
// the value according the current time t and the current phase x.
// t and x are members of class model.

void RightPart(valarray<double>& f)
{        
        f[0] = cos(t + 1);  // first equation is :  cos(t + 1)
        f[1] = x[0] + t;    // second equation is:  x1 + t
} 


// here we can declare additional variable of our model...
};  


int main(int argc, char* argv[])
{        

//********** 3-ST STEP: continue the 2-st step and do model
        SimpleModel MyFirstModel;  // now we can modeling 
        

//********** 4-ST STEP: preparing integration routine (Euler method) 
        Integration integr(0.1);  // parameter: step


//********** 5-ST STEP: preparing container for results 
        ResultManager result("result.txt"); // parameter: file with results


//********** 6-ST STEP: modeling from Cauchy condition x0=x(t0)
        valarray<double> x0(2);  
        x0[0] = 0.5;  // x1(t0)
        x0[1] = -0.5; // x2(t0)

        // modeling from t0=0 to T=10.
        integr.Calculate(MyFirstModel, CauchyRegion(0., x0, 10.), result);


        cout << "All done!  See file result.txt \n";

        return 0;
}
